#ifndef musicclock_h_
#define musicclock_h_

#include <avr/io.h>

#define USE_USB

#if defined(__AVR_AT90USB1286__)
#define led_on() PORTD |= _BV(6)
#define led_off() PORTD &= ~_BV(6)
#define led_tog() PORTD ^= _BV(6)
#define led_output() DDRD |= _BV(6)
#else
// atmega644 has no LEDs connected, so make these do nothing
#define led_on() _NOP()
#define led_off() _NOP()
#define led_tog() _NOP()
#define led_output() _NOP()
#endif

#ifndef assert
#define assert(x) if((x)==0){while(1){led_tog();_delay_ms(100);}}
#endif

#define printTwoNum(stream,x) if(x<10){fprintf((stream),"0%d",x);}else{fprintf((stream),"%d",x);}
#define printTwoNum_P(stream,x) if(x<10){fprintf_P((stream),PSTR("0%d"),x);}else{fprintf_P((stream),PSTR("%d"),x);}

typedef enum
{
    tosave_none = 0x00,
    tosave_ram = _BV(0),
    tosave_eeprom = _BV(1),
    tosave_time = _BV(2),
} tosave_t;

typedef enum
{
    playmode_normal,
    playmode_loop,
    playmode_shuffle,
    playmode_once,
    playmode_end,
} playmode_t;

typedef enum
{
    submenu_playMode = 0,
    submenu_timeMode, // 1
    submenu_usbMode, // 2
    submenu_currentTimeHour, // 3
    submenu_currentTimeMinute, // 4
    submenu_currentDayOfWeek, // 5
    submenu_alarms, // starts from 6 for monday alarm on/off, 4 + 7 * 3 = index for minute setting for sunday
    submenu_end = 27,
} submenu_t;

typedef enum
{
    timemode_twelve = 1, // AMPM mode
    timemode_twentyFour = 0,
} timemode_t;

typedef enum
{
    playstatus_ok, // played bytes
    playstatus_busy, // DREQ low at start
    playstatus_endOfFile, // reached end of file
}
mp3_playstatus_t;

typedef enum
{
    lcdshow_currentTime,
    lcdshow_menu,
    lcdshow_volume,
    lcdshow_other,
}
lcdcontent_t;

typedef enum { bool_false = 0, bool_true = 1, } bool_t;

#endif