#ifndef irctrl_h_
#define irctrl_h_

#include <inttypes.h>

// configuration
#define IRCTRL_PULSE_NUMOFONPULSES 34
#define IRCTRL_PULSE_STARTTHRESHOLD 8000 
#define IRCTRL_PULSE_ERROR 250 // allowed margin of error for each pulse
//#define IRCTRL_USEFLASH // do not enable, flash access is too slow
/*
 * Do not ever enable IRCTRL_USEFLASH
 * this IR command interpreter runs inside an ISR
 * through experimentation, I found that using codes stored in flash
 * memory caused the the function to run too slow for comfort
 *
 */

#include "pins.h"

// just some math macros
#define max(x,y) ((x)>(y)?(x):(y))
#define min(x,y) ((x)<(y)?(x):(y))

typedef enum
{
    ircmd_up = 0,
    ircmd_down,
    ircmd_left,
    ircmd_right,
    ircmd_play,
    ircmd_stop,
    ircmd_mute,
    ircmd_menu,
    ircmd_none,
}
ircmd_t;

ircmd_t irctrl_getCmd();

void irctrl_clearCmd();

uint8_t irctrl_isHeldDown();

#if defined(__AVR_AT90USB1286__)
#define IRCTRL_TIMER_CAPT_vect TIMER1_CAPT_vect
#define IRCTRL_TIMER_OVF_vect TIMER1_OVF_vect
#define IRCTRL_TIMER_ICR ICR1
#define IRCTRL_TIMER_TCCRxB TCCR1B
#define IRCTRL_TIMER_TIMSKx TIMSK1
#define IRCTRL_TIMER_ICIE ICIE1
#define IRCTRL_TIMER_ICIES ICES1
#define IRCTRL_TIMER_TOIE TOIE1
#elif defined(__AVR_ATmega644P__) || defined(__AVR_ATmega644__)
#define IRCTRL_TIMER_CAPT_vect TIMER1_CAPT_vect
#define IRCTRL_TIMER_OVF_vect TIMER1_OVF_vect
#define IRCTRL_TIMER_ICR ICR1
#define IRCTRL_TIMER_TCCRxB TCCR1B
#define IRCTRL_TIMER_TIMSKx TIMSK1
#define IRCTRL_TIMER_ICIE ICIE1
#define IRCTRL_TIMER_ICIES ICES1
#define IRCTRL_TIMER_TOIE TOIE1
#endif

#endif