#ifndef mp3_h_
#define mp3_h_

#include <inttypes.h>

#include "pins.h"

// macros to change SPI speed
#define spi_slowest() SPCR |= _BV(SPR1) | _BV(SPR0); SPSR &= ~_BV(SPI2X)
#define spi_slower() SPCR = (SPCR & (~_BV(SPR1))) | _BV(SPR0); SPSR |= _BV(SPI2X)
#define spi_slow() SPCR &= ~_BV(SPR1) & ~_BV(SPR0); SPSR &= ~_BV(SPI2X)
#define spi_fast() SPCR &= ~_BV(SPR1) & ~_BV(SPR0); SPSR |= _BV(SPI2X)

typedef enum
{
    // refer to datasheet
    MP3_REG_MODE = 0x00,
    MP3_REG_CLOCKF = 0x03,
    MP3_REG_VOL = 0x0B,
    MP3_REG_DECODE_TIME = 0x04,
}
mp3_register_t;

inline void mp3_waitReady(uint8_t cnt);

inline uint8_t mp3_spi(uint8_t data);

void mp3_writeReg(mp3_register_t addr, uint16_t data);

uint16_t mp3_readReg(mp3_register_t addr);

// specify volume from 1-255 with 255 being loudest
void mp3_setVolume(uint8_t volume);

void mp3_resetPlayTime();

uint16_t mp3_getPlayTime();

void mp3_playPacket(uint8_t* buffer, uint8_t length);

void mp3_startSineTest();

void mp3_keepSineTest();

void mp3_stopSineTest();

void mp3_init();

void mp3_powerUp();

void mp3_powerDown();

uint8_t mp3_isPowered();

#endif