#include "Lib/twi.h"
#include "rtc.h"
#include <avr/io.h>

// supply a pointer to the value you want to retrieve, or null to not retrieve that value
// this burst read method is faster
void rtc_getTimeBurst(uint8_t* year, uint8_t* month, uint8_t* day, uint8_t* dayOfWeek, uint8_t* hour, uint8_t* minute, uint8_t* second)
{
    uint8_t readCnt = 7;
    
    if (year == 0)
    {
        readCnt--;
        if (month == 0)
        {
            readCnt--;
            if (day == 0)
            {
                readCnt--;
            }
        }
    }
    
    uint8_t buffer[7];
    buffer[0] = 0x00; // set initial read pointer
    twi_writeTo(DS1307_ADDR, buffer, 1, 1);
    twi_readFrom(DS1307_ADDR, buffer, readCnt);
    
    if (year != 0)
    {
        *year = bcd_decode(buffer[6]);
    }
    
    if (month != 0)
    {
        *month = bcd_decode(buffer[5]);
    }
    
    if (day != 0)
    {
        *day = bcd_decode(buffer[4]);
    }
    
    if (dayOfWeek != 0)
    {
        *dayOfWeek = bcd_decode(buffer[3]);
    }
    
    if (hour != 0)
    {
        if (bit_is_set(buffer[2], 6))
        { // 12 hour mode
            if (bit_is_set(buffer[2], 5))
            { // PM
                *hour = bcd_decode(buffer[2] & 0b00011111) + 12;
            }
            else
            { // AM
                *hour = bcd_decode(buffer[2] & 0b00011111);
            }
        }
        else
        { // 24 hour mode
            *hour = bcd_decode(buffer[2] & 0b00111111);
        }
    }
    
    if (minute != 0)
    {
        *minute = bcd_decode(buffer[1]);
    }
    
    if (second != 0)
    {
        *second = bcd_decode(buffer[0] & 0x7F);
    }
}

// supply a pointer to the value you want to retrieve, or null to not retrieve that value
void rtc_getTime(uint8_t* year, uint8_t* month, uint8_t* day, uint8_t* dayOfWeek, uint8_t* hour, uint8_t* minute, uint8_t* second)
{
    uint8_t buffer[1];
    
    if (year != 0)
    {
        buffer[0] = 0x06; // set ds1307 internal register address
        twi_writeTo(DS1307_ADDR, buffer, 1, 1);
        twi_readFrom(DS1307_ADDR, buffer, 1);
        *year = bcd_decode(buffer[0]);
    }
    
    if (month != 0)
    {
        buffer[0] = 0x05; // set ds1307 internal register address
        twi_writeTo(DS1307_ADDR, buffer, 1, 1);
        twi_readFrom(DS1307_ADDR, buffer, 1);
        *month = bcd_decode(buffer[0]);
    }
    
    if (day != 0)
    {
        buffer[0] = 0x04; // set ds1307 internal register address
        twi_writeTo(DS1307_ADDR, buffer, 1, 1);
        twi_readFrom(DS1307_ADDR, buffer, 1);
        *day = bcd_decode(buffer[0]);
    }
    
    if (dayOfWeek != 0)
    {
        buffer[0] = 0x03; // set ds1307 internal register address
        twi_writeTo(DS1307_ADDR, buffer, 1, 1);
        twi_readFrom(DS1307_ADDR, buffer, 1);
        *dayOfWeek = bcd_decode(buffer[0]);
    }
    
    if (hour != 0)
    {
        buffer[0] = 0x02; // set ds1307 internal register address
        twi_writeTo(DS1307_ADDR, buffer, 1, 1);
        twi_readFrom(DS1307_ADDR, buffer, 1);
        if (bit_is_set(buffer[0], 6))
        { // 12 hour mode
            if (bit_is_set(buffer[0], 5))
            { // PM
                *hour = bcd_decode(buffer[0] & 0b00011111) + 12;
            }
            else
            { // AM
                *hour = bcd_decode(buffer[0] & 0b00011111);
            }
        }
        else
        { // 24 hour mode
            *hour = bcd_decode(buffer[0] & 0b00111111);
        }
    }
    
    if (minute != 0)
    {
        buffer[0] = 0x01; // set ds1307 internal register address
        twi_writeTo(DS1307_ADDR, buffer, 1, 1);
        twi_readFrom(DS1307_ADDR, buffer, 1);
        *minute = bcd_decode(buffer[0]);
    }
    
    if (second != 0)
    {
        buffer[0] = 0x00; // set ds1307 internal register address
        twi_writeTo(DS1307_ADDR, buffer, 1, 1);
        twi_readFrom(DS1307_ADDR, buffer, 1);
        *second = bcd_decode(buffer[0] & 0x7F);
    }
}

// quick time setting function
void rtc_setTimeBurst(uint8_t dayOfWeek, uint8_t hour, uint8_t minute, uint8_t second)
{
    uint8_t buffer[5];
    buffer[0] = 0x00;  // set initial write pointer
    buffer[1] = bcd_encode(second);
    buffer[2] = bcd_encode(minute);
    buffer[3] = bcd_encode(hour % 24);
    buffer[4] = bcd_encode(dayOfWeek);
    twi_writeTo(DS1307_ADDR, buffer, 5, 0);
}

// supply a pointer to the value you want to set, or null to not set that value
void rtc_setTime(uint8_t* year, uint8_t* month, uint8_t* day, uint8_t* dayOfWeek, uint8_t* hour, uint8_t* minute, uint8_t* second)
{
    uint8_t buffer[2];
    
    if (year != 0)
    {
        buffer[0] = 0x06; // set ds1307 internal register address
        buffer[1] = bcd_encode(*year);
        twi_writeTo(DS1307_ADDR, buffer, 2, 1);
    }
    
    if (month != 0)
    {
        buffer[0] = 0x05; // set ds1307 internal register address
        buffer[1] = bcd_encode(*month);
        twi_writeTo(DS1307_ADDR, buffer, 2, 1);
    }
    
    if (day != 0)
    {
        buffer[0] = 0x04; // set ds1307 internal register address
        buffer[1] = bcd_encode(*day);
        twi_writeTo(DS1307_ADDR, buffer, 2, 1);
    }
    
    if (dayOfWeek != 0)
    {
        buffer[0] = 0x03; // set ds1307 internal register address
        buffer[1] = bcd_encode(*dayOfWeek);
        twi_writeTo(DS1307_ADDR, buffer, 2, 1);
    }
    
    if (hour != 0)
    {        
        buffer[0] = 0x02; // set ds1307 internal register address
        buffer[1] = bcd_encode((*hour) % 24);
        twi_writeTo(DS1307_ADDR, buffer, 2, 1);
    }
    
    if (minute != 0)
    {
        buffer[0] = 0x01; // set ds1307 internal register address
        buffer[1] = bcd_encode(*minute);
        twi_writeTo(DS1307_ADDR, buffer, 2, 1);
    }
    
    if (second != 0)
    {
        buffer[0] = 0x00; // set ds1307 internal register address
        buffer[1] = bcd_encode(*second);
        twi_writeTo(DS1307_ADDR, buffer, 2, 1);
    }
}

void rtc_saveRam(uint8_t* data, int length)
{
    // writes to RAM inside DS1307 in chunks of 6 (because 1 address + 1 pointer + 6 bytes = 8 bytes total)
    uint8_t buffer[7];
    uint8_t checksum = 0;
    int i, j;
    for (i = 0, j = 0; i < length; i++)
    {
        buffer[j + 1] = data[i];
        checksum += data[i];
        
        if (j >= 5)
        {
            buffer[0] = i - j + RTC_USERRAMSTART; // set register pointer to user RAM
            twi_writeTo(DS1307_ADDR, buffer, 7, 1);
            j = 0;
        }
        else
        {
            j++;
        }
    }
    
    buffer[0] = i - j + RTC_USERRAMSTART; // set register pointer to user RAM
    buffer[j + 1] = checksum;
    twi_writeTo(DS1307_ADDR, buffer, j + 2, 1);
}

uint8_t rtc_loadRam(uint8_t* data, int length)
{
    // loads data from DS1307 user RAM in chunks of 7

    uint8_t buffer[7];
    
    // set read pointer first
    buffer[0] = RTC_USERRAMSTART;
    twi_writeTo(DS1307_ADDR, buffer, 1, 1);
    
    int totalReadCnt = 0;
    int thisReadCnt = 0;
    int idx = 0;
    int i = 0;
    while (totalReadCnt < length + 1)
    {
        totalReadCnt += (thisReadCnt = twi_readFrom(DS1307_ADDR, buffer, 7));
        for (i = 0; i < thisReadCnt && idx < length + 1; i++, idx++)
        {
            data[idx] = buffer[i];
        }
    }
    
    return buffer[length % 7]; // return the checksum
}

// performs a test to see if information is saved properly, check logic analyzer waveforms to verify results;
void rtc_ramTest()
{
    uint8_t buffer[0x3F - 0x08];
    uint8_t checksum1;
    for (int amount = 1; amount < 0x3F - 0x08 - 2; amount++)
    {
        checksum1 = 0;
        for (int i = 0; i < amount; i++)
        {
            buffer[i] = i;
            checksum1 += i;
        }
        
        rtc_saveRam(buffer, amount);
        uint8_t newChecksum = rtc_loadRam(buffer, amount);
        
        uint8_t checksum2 = 0;
        for (int i = 0; i < amount; i++)
        {
            checksum2 += buffer[i];
        }
        
        buffer[0] = RTC_USERRAMSTART;
        buffer[1] = checksum1;
        buffer[2] = checksum2;
        buffer[3] = newChecksum;
        
        twi_writeTo(DS1307_ADDR, buffer, 4, 1);
    }
}

void rtc_run()
{
    // the only purpose of this function is to clear
    // the CH bit in the DS1307 registers
    // thus allowing the RTC to run

    uint8_t tmpSec = 0;
    rtc_getTime(0, 0, 0, 0, 0, 0, &tmpSec);
    rtc_setTime(0, 0, 0, 0, 0, 0, &tmpSec);
}

void rtc_init()
{
    // port must be set as input but with internal pull-up resistors enabled
    TWI_PORTOUTPUT |= _BV(TWI_PIN_SDA);
    TWI_PORTOUTPUT |= _BV(TWI_PIN_SCL);
    TWI_PORTDIR &= ~_BV(TWI_PIN_SDA);
    TWI_PORTDIR &= ~_BV(TWI_PIN_SCL);
    
    // start TWI
    twi_init();
}