#include <avr/eeprom.h>

void loadFromEeprom()
{
    uint8_t checksum = 0, i = 0;

    uint8_t* p = &eepromSettings;
    
    // read in entire block while tracking checksum
    for (i = 0; i < sizeof(eepromSettings); i++)
    {
        uint8_t data = eeprom_read_byte(i);
        p[i] = data;
        checksum += data;
    }
    
    // reset values if checksum failed
    if (eeprom_read_byte(i) != checksum)
    {
        eepromSettings.alarmMask = 0;
        for (i = 0; i < 7; i++)
        {
            eepromSettings.alarmHour[i] = 8;
            eepromSettings.alarmMinute[i] = 0;
        }
    }
}

void saveToEeprom(uint8_t* data, int length)
{
    // write entire block to eeprom, ending it with the checksum
    int i;
    uint8_t checksum = 0;
    for (i = 0; i < length; i++)
    {
        checksum += data[i];
        eeprom_update_byte(i, data[i]);
    }
    eeprom_update_byte(i, checksum);
}

void loadFromRam()
{
    // get data first
    uint8_t checksum = rtc_loadRam(&ramSettings, sizeof(ramSettings));

    uint8_t* p = &ramSettings;

    // calculate appropriate checksum
    for (int i = 0; i < sizeof(ramSettings); i++)
    {
        checksum -= p[i];
    }
    
    // checksum does not match, reset to defaults
    if (checksum != 0)
    {
        ramSettings.volume = 0xFF;
        ramSettings.mute = bool_false;
        ramSettings.playMode = playmode_normal;
        ramSettings.submenu = 0;
        ramSettings.timeMode = timemode_twelve;
    }
}