#include <string.h>
#include <stdlib.h>
#include <inttypes.h>

/*
 * this code will try to alphabetically sorts a limited number of files
 * if too many files are found, it should switch to non-sorted mode
 * since we don't have that much RAM
 * 
 * I'm not sure if this section actually works, but it's fail safe
 *
 * I tried runtime memory allocation using malloc/calloc/free
 * but that didn't work so well so now the allocation is fixed
 *
 */
#define MAX_NUM_FILES 64

typedef char filename_t[8 + 1 + 3 + 1];

static uint16_t numOfFiles = 0;
static filename_t fileNameList[MAX_NUM_FILES];
static bool_t useSortedList;
static int16_t curSongIdx = -1; // start at -1 so the first call to openNextFile
// will start at 0

void songListInit(DIR* tdir)
{
    useSortedList = bool_false;
    
    numOfFiles = 0;
    FILINFO tfileInfo;
    bool_t failedOnce = bool_false;
    
    wdt_reset();

    // count files and copy file names into the list
    err = f_opendir(tdir, "/"); // start over
    failedOnce = bool_false;
    int cnt = 0;
    while (cnt < MAX_NUM_FILES)
    {
        err = f_readdir(tdir, &tfileInfo);
        if (err == FR_OK)
        {
            if (tfileInfo.fname[0] != 0)
            {
                if ((tfileInfo.fattrib & AM_DIR) == 0)
                {
                    strcpy(fileNameList[cnt], tfileInfo.fname);
                    cnt++;
                }
            }
            else
            {
                // end of list
                break;
            }
        }
        else if (failedOnce == bool_false)
        {
            // first failure, retry
            err = f_opendir(tdir, "/"); // start over
            if (err != FR_OK) break;
            cnt = 0;
            failedOnce = bool_true;
        }
        else
        {
            // already failed once
            break;
        }
    }

    wdt_reset();
    
    // quick sort the file names
    qsort(fileNameList, cnt, sizeof(filename_t), strcmp);
    
    // finalize the initialization
    numOfFiles = cnt;
    if (cnt > 0 && cnt < MAX_NUM_FILES)
    {
        useSortedList = bool_true;
    }
}

bool_t openNextFile(DIR* dir, FILINFO* fileInfo, FIL* file, char* fileName)
{
    if (useSortedList != bool_false)
    {
        curSongIdx = (curSongIdx + 1) % numOfFiles;
        char path[1 + 8 + 1 + 3 + 2] = { '/',
        fileNameList[curSongIdx][0],
        fileNameList[curSongIdx][1],
        fileNameList[curSongIdx][2],
        fileNameList[curSongIdx][3],
        fileNameList[curSongIdx][4],
        fileNameList[curSongIdx][5],
        fileNameList[curSongIdx][6],
        fileNameList[curSongIdx][7],
        fileNameList[curSongIdx][8],
        fileNameList[curSongIdx][9],
        fileNameList[curSongIdx][10],
        fileNameList[curSongIdx][11],
        fileNameList[curSongIdx][12], // should be 0 too
        0, }; // this copies the name to a path to use
        err = f_open(file, path, FA_READ);
        strcpy(fileName, fileNameList[curSongIdx]);
        return err == FR_OK;
    }

    bool_t looped = bool_false;
    bool_t failedOnce = bool_false;

    while (1)
    {
        err = f_readdir(dir, fileInfo); // next file

        if (err != FR_OK && failedOnce == bool_false)
        {
            // first failure, retry
            err = f_opendir(dir, "/");
            if (err != FR_OK) return bool_false;
            failedOnce = bool_true;
        }
        else
        // check if disk is ok, file exists, and is not a directory
        if (err == FR_OK && fileInfo->fname[0] != 0 && (fileInfo->fattrib & AM_DIR) == 0)
        {
            char fpath[1 + 8 + 1 + 3 + 2] = { '/', fileInfo->fname[0], fileInfo->fname[1], fileInfo->fname[2], fileInfo->fname[3], fileInfo->fname[4], fileInfo->fname[5], fileInfo->fname[6], fileInfo->fname[7], fileInfo->fname[8], fileInfo->fname[9], fileInfo->fname[10], fileInfo->fname[11], fileInfo->fname[12], 0, }; // this copies the name to a path to use
            
            // if exists and is not a directory
            err = f_open(file, fpath, FA_READ);
            if (err == FR_OK)
            {
                strcpy(fileName, fileInfo->fname);
                return bool_true;
            }
        }
        else if (err == FR_OK && fileInfo->fname[0] == 0)
        {
            // end of files, restart from the beginning
            
            err = f_opendir(dir, "/");
            
            if (err != FR_OK) return bool_false;
            
            if (looped != bool_false) return bool_false; // restarted once already, so this means no files found
            
            looped = bool_true; // indicate we have gone to the end once
        }
        else if (err != FR_OK)
        {
             return bool_false; // can't read from disk
        }
    }
}

bool_t openPrevFile(DIR* dir, FILINFO* fileInfo, FIL* file, char* fileName)
{
    if (useSortedList != bool_false)
    {
        curSongIdx = curSongIdx <= 0 ? numOfFiles - 1 : curSongIdx - 1;
        char path[1 + 8 + 1 + 3 + 2] = { '/',
        fileNameList[curSongIdx][0],
        fileNameList[curSongIdx][1],
        fileNameList[curSongIdx][2],
        fileNameList[curSongIdx][3],
        fileNameList[curSongIdx][4],
        fileNameList[curSongIdx][5],
        fileNameList[curSongIdx][6],
        fileNameList[curSongIdx][7],
        fileNameList[curSongIdx][8],
        fileNameList[curSongIdx][9],
        fileNameList[curSongIdx][10],
        fileNameList[curSongIdx][11],
        fileNameList[curSongIdx][12], // should be 0 too
        0, }; // this copies the name to a path to use
        err = f_open(file, path, FA_READ);
        strcpy(fileName, fileNameList[curSongIdx]);
        return err == FR_OK;
    }

    // find the file to open by matching the current file, and returning the one before it.
    
    DIR tDir;
    FILINFO nFileInfo;
    FILINFO pFileInfo;
    FIL nFile;
    FIL pFile;
    
    uint8_t startSecond = currentTime.second;
    
    memcpy(&tDir, dir, sizeof(DIR));
    memcpy(&nFileInfo, fileInfo, sizeof(FILINFO));
    memcpy(&pFileInfo, fileInfo, sizeof(FILINFO));
    memcpy(&nFile, file, sizeof(FIL));
    memcpy(&pFile, file, sizeof(FIL));
    
    while (1)
    {
        bool_t err = openNextFile(&tDir, &nFileInfo, &nFile, fileName);
        if (err == bool_false) return err;
        
        if (strcmp(nFileInfo.fname, fileInfo->fname) == 0)
        {
            // found match, so the previous entry is copied back
            memcpy(fileInfo, &pFileInfo, sizeof(FILINFO));
            memcpy(file, &pFile, sizeof(FIL));
            return bool_true;
        }
        else
        {
            // not a match so store previous copy
            memcpy(&pFileInfo, &nFileInfo, sizeof(FILINFO));
            memcpy(&pFile, &nFile, sizeof(FIL));
        }
        
        // 5 second timeout
        uint8_t curSec = currentTime.second + 60;
        if ((curSec - startSecond) % 60 > 5)
        {
            return bool_false;
        }

        wdt_reset();
    }
}

// open the next file a random amount of time for shuffle mode
bool_t openRandomFile(DIR* dir, FILINFO* fileInfo, FIL* file, char* fileName)
{
    if (useSortedList != bool_false)
    {
        curSongIdx = rand() % numOfFiles;
        char path[1 + 8 + 1 + 3 + 2] = { '/',
        fileNameList[curSongIdx][0],
        fileNameList[curSongIdx][1],
        fileNameList[curSongIdx][2],
        fileNameList[curSongIdx][3],
        fileNameList[curSongIdx][4],
        fileNameList[curSongIdx][5],
        fileNameList[curSongIdx][6],
        fileNameList[curSongIdx][7],
        fileNameList[curSongIdx][8],
        fileNameList[curSongIdx][9],
        fileNameList[curSongIdx][10],
        fileNameList[curSongIdx][11],
        fileNameList[curSongIdx][12], // should be 0 too
        0, }; // this copies the name to a path to use
        err = f_open(file, path, FA_READ);
        strcpy(fileName, fileNameList[curSongIdx]);
        return err == FR_OK;
    }

    bool_t err;
    
    int toIterate = (rand() & 0x0F) + 1;
    for (int i = 0; i < toIterate; i++)
    {
        err = openNextFile(dir, fileInfo, file, fileName);
        if (err == bool_false) return bool_false;

        wdt_reset();
    }
    
    return err;
}